interface Storage {
    slot: string;
    offset: number;
    type: string;
    label: string;
}
interface Types {
    [type: string]: {
        encoding: string;
        numberOfBytes: string;
        label: string;
        base?: string;
        value?: string;
        key?: string;
        members?: Storage[];
    };
}
export type MappingKey = string | boolean | number;
export interface StorageLayout {
    storage: Storage[];
    types: Types;
}
export interface StorageInfo extends Storage {
    types: Types;
}
export type GetStorageAt = (param: {
    blockHash: string;
    contract: string;
    slot: string;
}) => Promise<{
    value: string;
    proof: {
        data: string;
    };
}>;
/**
 * Function to get storage information of variable from storage layout.
 * @param storageLayout
 * @param variableName
 */
export declare const getStorageInfo: (storageLayout: StorageLayout, variableName: string) => StorageInfo;
/**
 * Function to get the value from storage for a contract variable.
 * @param storageLayout
 * @param getStorageAt
 * @param blockHash
 * @param address
 * @param variableName
 */
export declare const getStorageValue: (storageLayout: StorageLayout, getStorageAt: GetStorageAt, blockHash: string, address: string, variableName: string, ...mappingKeys: Array<MappingKey>) => Promise<{
    value: any;
    proof: {
        data: string;
    };
}>;
/**
 * Get value according to type described by the label.
 * @param storageValue
 * @param typeLabel
 */
export declare const getValueByType: (storageValue: string, typeLabel: string) => bigint | string | boolean;
/**
 * Function to get slot for mapping types.
 * @param mappingSlot
 * @param key
 */
export declare const getMappingSlot: (types: Types, mappingSlot: string, keyType: string, key: MappingKey) => string;
export {};
